// Thermo.c
//
// Routinen zur Ansteuerung des DS18S20 Thermometers
//

#include "Thermo.h"
#include "MyUtils.h"
#include "nokialcd.h"

static signed char TemperaturAlt = 0;
static bool TemperaturGelesen = false;

// Initialisiert eine Kommunikation auf dem Bus
bool THERMO_InitBus( void )
{
	bool OK = false;
	unsigned char i;
	
	// Reset Signal senden
	TH_BS;
	
	TH_BL;
	Delayms(0.6);
	TH_BH;
	//Delayms(0.02);	//UN
	// Auf Antwort von DS18S20 warten
	TH_BR;
	for (i = 0; i < 48;i++)
	{
		Delayms(0.01);
		if (!TH_RB)
			OK = true;
	};

	// Bus wieder zurcksetzen	
	TH_BS;
	TH_BH;
	
	return OK;
};

// Sendet ein Bit
void THERMO_WriteBit( bool Bit )
{
   // Schreibkommando senden
	TH_BS;
	TH_BL;
	Delayms(0.01);	
	// Bit schreiben
	if (Bit)
		TH_BH;
	Delayms(0.06);

	// bertragung beenden
	TH_BH;
	Delayms(0.01);
};

// Schreibt ein Byte auf den Bus
void THERMO_WriteByte(unsigned char Byte)
{
	unsigned char i = 0;
	for (i = 0; i < 8; i++)
	{
		THERMO_WriteBit(Byte & (unsigned char) 0x01);
		Byte >>= 1;
	};
};

// Liest ein Bit
bool THERMO_ReadBit( void )
{
	bool Result;
	
	// Lesekommando senden
	TH_BS;
	TH_BL;
	Delayms(0.002);	
	TH_BH;
	TH_BR;
	Delayms(0.004);	

	// Bus lesen
	Result = TH_RB;

	Delayms(0.06);
	// Bus zurcksetzen
	TH_BS;
	Delayms(0.01);

	return Result;	
};

// Liest ein Byte vom Bus
unsigned char THERMO_ReadByte( void )
{
	unsigned char Result = 0;
	unsigned char i = 0;
	
	for (i = 0; i < 8; i++)
	{
		Result >>= 1;
		Result |= (THERMO_ReadBit() ? 0x80 : 0) ;
	};
	
	return Result;
};

// Liest Daten vom Bus
void THERMO_ReadData(unsigned char *Buffer, unsigned char Count)
{
unsigned char dummy;
	while (Count--)
	{
		//dummy = THERMO_ReadBit();
		*Buffer = THERMO_ReadByte();
		Buffer++;	
	};
};

// Gibt den Befehl die Temperatur zu lesen
void THERMO_Init_Temp_Read( void )
{
	// Feststellen, ob Temperatur noch gemessen wird
	/*if (TemperaturGelesen && !THERMO_ReadBit())
		return;*/
	
	// Bus initialisieren
	THERMO_InitBus();
	
	// Ziel whlen
	THERMO_WriteByte(TH_SKIP_ROM);
	
	// Messen Befehl senden
	THERMO_WriteByte(TH_CONVERT_TEMPERATURE);	
	
	TemperaturGelesen = true;
};

unsigned char CRCTable[] =
{
0, 94, 188, 226, 97, 63, 221, 131, 194, 156, 126, 32, 163, 253, 31, 65,
157, 195, 33, 127, 252, 162, 64, 30, 95, 1, 227, 189, 62, 96, 130, 220,
35, 125, 159, 193, 66, 28, 254, 160, 225, 191, 93, 3, 128, 222, 60, 98,
190, 224, 2, 92, 223, 129, 99, 61, 124, 34, 192, 158, 29, 67, 161, 255,
70, 24, 250, 164, 39, 121, 155, 197, 132, 218, 56, 102, 229, 187, 89, 7,
219, 133, 103, 57, 186, 228, 6, 88, 25, 71, 165, 251, 120, 38, 196, 154,
101, 59, 217, 135, 4, 90, 184, 230, 167, 249, 27, 69, 198, 152, 122, 36,
248, 166, 68, 26, 153, 199, 37, 123, 58, 100, 134, 216, 91, 5, 231, 185,
140, 210, 48, 110, 237, 179, 81, 15, 78, 16, 242, 172, 47, 113, 147, 205,
17, 79, 173, 243, 112, 46, 204, 146, 211, 141, 111, 49, 178, 236, 14, 80,
175, 241, 19, 77, 206, 144, 114, 44, 109, 51, 209, 143, 12, 82, 176, 238,
50, 108, 142, 208, 83, 13, 239, 177, 240, 174, 76, 18, 145, 207, 45, 115,
202, 148, 118, 40, 171, 245, 23, 73, 8, 86, 180, 234, 105, 55, 213, 139,
87, 9, 235, 181, 54, 104, 138, 212, 149, 203, 41, 119, 244, 170, 72, 22,
233, 183, 85, 11, 136, 214, 52, 106, 43, 117, 151, 201, 74, 20, 246, 168,
116, 42, 200, 150, 21, 75, 169, 247, 182, 232, 10, 84, 215, 137, 107, 53};

	unsigned char Data[20];
// Liest die Temperatur ein
signed char THERMO_GetTemp( void )
{
	unsigned char i = 0;
	unsigned char CRC = 0;
	
	TemperaturGelesen = false;
	THERMO_Init_Temp_Read();
	
	//while (!THERMO_ReadBit());
	TH_BS;
	TH_BH;

	for (i = 0; i < 0xFF;i++)
		Delayms(4.0);
	for (i = 0; i < 0xFF;i++)
		Delayms(4.0);
	
	LcdGotoXY(1,5);
	for (i = 0; i <9;i++){
		LcdHex(Data[i]);
	}
	LcdUpdate();

	// Feststellen, ob Temperaturdaten bereit sind
	/*if 	(TemperaturGelesen && !THERMO_ReadBit())
		return TemperaturAlt;*/
		
	TemperaturGelesen = false;

	// Bus initialisieren
	if (!THERMO_InitBus()){
		LcdGotoXY(1,5);
		LcdStr( FONT_1X, "init99");
		return -99;
	}


	// Ziel whlen
	THERMO_WriteByte(TH_SKIP_ROM);
	
	// Messen Befehl senden
	THERMO_WriteByte(TH_READ_SCRATCH_PAD);
	
	// Daten lesen
	THERMO_ReadData(Data,9);
	return 0;


	for (i = 0; i <8;i++){
		CRC = CRCTable[CRC ^ Data[i]];
	}

	if (CRC != Data[8])
	{
		LcdGotoXY(1,5);
		LcdStr( FONT_1X, "CRC Error");
		LcdUpdate();
//		LCD_ShowText("CRC Error");
//		for(;;);
	};

	TemperaturAlt =  Data[0];//(Data[0]  >> 1 | Data[1] << 7);
	
	return (signed char) TemperaturAlt;

};
